@@ -16,9 +16,11 @@ import qualified Network.HTTP.Conduit as H
 import Network.HTTP.Types
 import Network.Wai
 import Network.Wai.Application.Classic.Conduit
+import Network.Wai.Application.Classic.EventSource
 import Network.Wai.Application.Classic.Field
 import Network.Wai.Application.Classic.Path
 import Network.Wai.Application.Classic.Types
+import Blaze.ByteString.Builder (Builder)
 import Prelude hiding (catch)
 
 toHTTPRequest :: Request -> RevProxyRoute -> Int64 -> H.Request (ResourceT IO)
@@ -53,7 +55,7 @@ getBody req len = H.RequestBodySource len (toBodySource req)
 
 getLen :: Request -> Maybe Int64
 getLen req = do
-    len' <- lookup "content-length" $ requestHeaders req
+    len' <- lookup hContentLength $ requestHeaders req
     case reads $ BS.unpack len' of
         [] -> Nothing
         (i, _):_ -> Just i
@@ -72,18 +74,25 @@ revProxyApp' cspec spec route req = do
     let mlen = getLen req
         len = fromMaybe 0 mlen
         httpReq = toHTTPRequest req route len
-    H.Response status _ hdr downbody <- http httpReq mgr
+    H.Response status _ hdr rdownbody <- http httpReq mgr
     let hdr' = fixHeader hdr
     liftIO $ logger cspec req status (fromIntegral <$> mlen)
-    return $ ResponseSource status hdr' (toResponseSource downbody)
+    ResponseSource status hdr' <$> toSource (lookup hContentType hdr') rdownbody
   where
     mgr = revProxyManager spec
     fixHeader = addVia cspec req . filter p
-    p ("Content-Encoding", _) = False
-    p ("Content-Length", _)   = False
-    p _ = True
+    p (k,_)
+      | k == hContentEncoding = False
+      | k == hContentLength   = False
+      | otherwise              = True
 
-type Resp = ResourceT IO (H.Response (Source (ResourceT IO) BS.ByteString))
+toSource :: Maybe BS.ByteString
+         -> ResumableSource (ResourceT IO) BS.ByteString
+         -> (ResourceT IO) (Source (ResourceT IO) (Flush Builder))
+toSource (Just "text/event-stream") = toResponseEventSource
+toSource _                          = toResponseSource
+
+type Resp = ResourceT IO (H.Response (ResumableSource (ResourceT IO) BS.ByteString))
 
 http :: H.Request (ResourceT IO) -> H.Manager -> Resp
 http req mgr = H.http req mgr
